@@ -621,7 +621,9 @@ pub trait KUtility{
   /// See the example of [`error_to_string`](fn.error_to_string.html).
   fn get_error_string(&self) -> Result<&str, &'static str>;
 
-  /// Get a table row of the given index.
+  /// Get a table row of the given index. For enumerated column, a names of a target `sym` list
+  ///  to which symbol values are cast must be passed. In the example below, it is assumed that
+  ///  there is a single enum column in a table and the column values are cast to a symbol list whose name is `sym`.
   /// ```no_run
   /// use kdbplus::api::*;
   /// 
@@ -629,15 +631,15 @@ pub trait KUtility{
   /// pub extern "C" fn print_row(object: K, index: K) -> K{
   ///   match object.get_type(){
   ///     qtype::TABLE => {
-  ///       if let Some(row) = object.get_row("sym", index.get_long().unwrap() as usize){
-  ///         let null = unsafe{k(0, str_to_S!("{-1 \"row: \", .Q.s1 x}"), row, KNULL)};
-  ///         decrement_reference_count(null);
-  ///         KNULL
+  ///       match object.get_row(&["sym"], index.get_long().unwrap() as usize){
+  ///         Ok(row) => {
+  ///           let null = unsafe{k(0, str_to_S!("{-1 \"row: \", .Q.s1 x}"), row, KNULL)};
+  ///           decrement_reference_count(null);
+  ///           KNULL
+  ///         }
+  ///         Err(error) => new_error(error)
   ///       }
-  ///       else{
-  ///         new_error("index out of bounds\0")
-  ///       }
-  ///     }
+  ///     },
   ///     _ => new_error("not a table\0")
   ///   }
   /// }
@@ -650,7 +652,7 @@ pub trait KUtility{
   /// q)row[table;1]
   /// row: `time`sym`go`miscellaneous!(2022.01.30D07:55:47.987133353;`Green;"x";`lion)
   /// ```
-  fn get_row(&self, sym: &str, index: usize) -> Option<K>;
+  fn get_row(&self, enum_sources: &[&str], index: usize) -> Result<K, &'static str>;
 
   /// Get an attribute of a q object.
   /// # Example
@@ -944,19 +946,20 @@ impl KUtility for K{
     }
   }
 
-  fn get_row(&self, sym: &str, index: usize) -> Option<K>{
+  fn get_row(&self, enum_sources: &[&str], index: usize) -> Result<K, &'static str>{
     match unsafe{(**self).qtype}{
       qtype::TABLE => {
         let keys = unsafe{(**self).value.table}.as_mut_slice::<K>()[0];
         let values = unsafe{(**self).value.table}.as_mut_slice::<K>()[1];
         if (unsafe{(*values.as_mut_slice::<K>()[0]).value.list}.n as usize) < index+1{
           // Index out of bounds
-          None
+          Err("index out of bounds\0")
         }
         else{
           let num_columns = unsafe{(*keys).value.list}.n;
           let row = new_list(qtype::COMPOUND_LIST, num_columns);
           let row_slice = row.as_mut_slice::<K>();
+          let mut enum_source_index = 0;
           let mut i = 0;
           for column in values.as_mut_slice::<K>(){
             match column.get_type(){
@@ -1012,13 +1015,22 @@ impl KUtility for K{
                 row_slice[i] = new_time(column.as_mut_slice::<I>()[index]);
               },
               qtype::ENUM_LIST => {
-                let enum_value = new_enum(sym, column.as_mut_slice::<J>()[index]);
-                if unsafe{(*enum_value).qtype} == qtype::ERROR{
+                if enum_sources.len() <= enum_source_index{
+                  // Index out of bounds
                   decrement_reference_count(row);
-                  return Some(enum_value);
+                  return Err("insufficient enum sources\0");
+                }
+                let enum_value = new_enum(enum_sources[enum_source_index], column.as_mut_slice::<J>()[index]);
+                if unsafe{(*enum_value).qtype} == qtype::ERROR{
+                  // Error in creating enum object.
+                  decrement_reference_count(row);
+                  let error = S_to_str(unsafe{(*enum_value).value.symbol});
+                  decrement_reference_count(enum_value);
+                  return Err(error);
                 }
                 else{
                   row_slice[i] = enum_value;
+                  enum_source_index += 1;
                 }
               }
               qtype::COMPOUND_LIST => {
@@ -1030,10 +1042,10 @@ impl KUtility for K{
             }
             i += 1;
           }
-          Some(new_dictionary(increment_reference_count(keys), row))
+          Ok(new_dictionary(increment_reference_count(keys), row))
         }
       },
-      _ => None
+      _ => Err("not a table\0")
     }
   }
 
@@ -1198,7 +1210,7 @@ impl KUtility for K{
   #[inline]
   fn push_raw<T>(&mut self, mut atom: T)->Result<K, &'static str>{
     match unsafe{(**self).qtype}{
-      _t@qtype::BOOL_LIST..=qtype::TIME_LIST => Ok(unsafe{native::ja(self, std::mem::transmute::<*mut T, *mut V>(&mut atom))}),
+      _t@qtype::BOOL_LIST..=qtype::ENUM_LIST => Ok(unsafe{native::ja(self, std::mem::transmute::<*mut T, *mut V>(&mut atom))}),
       _ => Err("not a simple list or types do not match\0")
     }
   }
@@ -1222,11 +1234,11 @@ impl KUtility for K{
   #[inline]
   fn len(&self) -> i64{
     match unsafe{(**self).qtype}{
-      _t@qtype::TIME_ATOM..=qtype::BOOL_ATOM => {
+      _t@qtype::ENUM_ATOM..=qtype::BOOL_ATOM => {
         // Atom
         1
       },
-      _t@qtype::COMPOUND_LIST..=qtype::TIME_LIST => {
+      _t@qtype::COMPOUND_LIST..=qtype::ENUM_LIST => {
         // List
         unsafe{(**self).value.list}.n
       },
@@ -2684,22 +2696,39 @@ pub fn days_to_ymd(days: I) -> I{
 ///   simple.as_mut_slice::<I>().copy_from_slice(&[12, 34]);
 ///   let extra=new_list(qtype::COMPOUND_LIST, 2);
 ///   extra.as_mut_slice::<K>().copy_from_slice(&[new_symbol("vague"), new_int(-3000)]);
-///   let mut compound = simple_to_compound(simple);
+///   let mut compound = simple_to_compound(simple, &[]);
 ///   compound.append(extra).unwrap()
 /// }
+/// 
+/// #[no_mangle]
+/// pub extern "C" fn drift2(_: K)->K{
+///   let simple=new_list(qtype::ENUM_LIST, 3);
+///   simple.as_mut_slice::<J>().copy_from_slice(&[0, 1, 2]);
+///   let mut compound = simple_to_compound(simple, &["enum", "enum2", "enum"]);
+///   compound.push(new_month(3)).unwrap();
+///   compound
+/// }
 /// ```
 /// ```q
 /// q)drift: LIBPATH_ (`drift; 1);
+/// q)drift2: LIBPATH_ (`drift2; 1);
 /// q)drift[]
 /// 12i
 /// 34i
 /// `vague
 /// -3000i
+/// q)enum: `mashroom`broccoli`cucumber
+/// q)enum2: `mackerel`swordfish
+/// q)drift2[]
+/// `enum$`mashroom
+/// `enum2$`swordfish
+/// `enum$`cucumber
+/// 2000.04m
 /// ```
 /// # Note
 /// To convert a list provided externally (i.e., passed from a q process), apply
 ///  [`increment_reference_count`](fn.increment_reference_count.html) before converting the list.
-pub fn simple_to_compound(simple: K) -> K{
+pub fn simple_to_compound(simple: K, enum_sources: &[&str]) -> K{
   let size=simple.len() as usize;
   let compound=new_list(qtype::COMPOUND_LIST, size as J);
   let compound_slice=compound.as_mut_slice::<K>();
@@ -2782,6 +2811,15 @@ pub fn simple_to_compound(simple: K) -> K{
         compound_slice[i]=new_time(simple_slice[i]);
       }
     },
+    qtype::ENUM_LIST => {
+      if enum_sources.len() < size{
+        return new_error("insufficient enum sources\0");
+      }
+      let simple_slice=simple.as_mut_slice::<J>();
+      for i in 0..size{
+        compound_slice[i]=new_enum(enum_sources[i], simple_slice[i]);
+      }
+    },
     _ => {
       decrement_reference_count(compound);
       return new_error("not a simple list\0");
